@@ -13,6 +13,10 @@ except ImportError:
 # from the list of packages for which version numbers are displayed
 # when running the tests.
 PYTEST_HEADER_MODULES['Astropy'] = 'astropy'
-PYTEST_HEADER_MODULES['scikit-image'] = 'skimage'
+PYTEST_HEADER_MODULES['photutils'] = 'photutils'
+PYTEST_HEADER_MODULES['Pillow'] = 'PIL'
+PYTEST_HEADER_MODULES['QtPy'] = 'qtpy'
+PYTEST_HEADER_MODULES.pop('h5py', None)
+PYTEST_HEADER_MODULES.pop('Pandas', None)
 
 TESTED_VERSIONS['ginga'] = version
