@@ -9,16 +9,16 @@ import numpy as np
 from ginga import trcalc
 from ginga.misc import Bunch
 #from ginga.misc.ParamSet import Param
-from ginga.util import zscale
+
+from astropy.visualization import ZScaleInterval
 
 have_scipy = True
-autocut_methods = ('minmax', 'median', 'histogram', 'stddev', 'zscale')
+autocut_methods = ['minmax', 'median', 'histogram', 'stddev', 'zscale']
 try:
     import scipy.ndimage.filters
 except ImportError:
     have_scipy = False
-    autocut_methods = ('minmax', 'histogram', 'stddev', 'zscale')
-
+    autocut_methods = ['minmax', 'histogram', 'stddev', 'zscale']
 
 __all__ = ['AutoCutsBase', 'Minmax', 'Histogram', 'StdDev', 'MedianFilter',
            'ZScale', 'get_autocuts', 'get_autocuts_names']
@@ -40,13 +40,14 @@ class AutoCutsBase(object):
         return []
 
     def __init__(self, logger):
-        super(AutoCutsBase, self).__init__()
+        super().__init__()
 
         self.logger = logger
         self.kind = 'base'
         self.crop_radius = 512
-        self.max_sample = 1000
+        self.max_sample = 20000
         self.pct_sample = 0.02
+        self.min_sample = 1000
 
     def update_params(self, **param_dict):
         # TODO: find a cleaner way to update these
@@ -175,7 +176,8 @@ class AutoCutsBase(object):
         wd, ht = image.get_size()
         total_points = wd * ht
         if num_points is None:
-            num_points = min(int(total_points * self.pct_sample),
+            num_points = min(max(self.min_sample,
+                                 int(total_points * self.pct_sample)),
                              self.max_sample)
         num_points = min(num_points, total_points)
         if num_points == 0:
@@ -213,7 +215,8 @@ class AutoCutsBase(object):
         ht, wd = data.shape[:2]
         total_points = wd * ht
         if num_points is None:
-            num_points = min(int(total_points * self.pct_sample),
+            num_points = min(max(self.min_sample,
+                                 int(total_points * self.pct_sample)),
                              self.max_sample)
         num_points = min(num_points, total_points)
         if num_points == 0:
@@ -324,19 +327,19 @@ class Clip(AutoCutsBase):
     """
 
     def __init__(self, logger):
-        super(Clip, self).__init__(logger)
+        super().__init__(logger)
         self.kind = 'clip'
 
     def calc_cut_levels(self, image):
         loval, hival = image.get_minmax()
 
-        return (float(loval), float(hival))
+        return float(loval), float(hival)
 
     def calc_cut_levels_data(self, data_np):
         loval = np.nanmin(data_np)
         hival = np.nanmax(data_np)
 
-        return (float(loval), float(hival))
+        return float(loval), float(hival)
 
     def cut_levels(self, data, loval, hival, vmin=0.0, vmax=255.0):
         return data.clip(vmin, vmax)
@@ -357,14 +360,14 @@ class Minmax(AutoCutsBase):
     """
 
     def __init__(self, logger):
-        super(Minmax, self).__init__(logger)
+        super().__init__(logger)
         self.kind = 'minmax'
 
     def calc_cut_levels(self, image):
         """See subclass documentation."""
         loval, hival = image.get_minmax()
 
-        return (float(loval), float(hival))
+        return float(loval), float(hival)
 
     def calc_cut_levels_data(self, data_np):
         """See subclass documentation."""
@@ -374,7 +377,7 @@ class Minmax(AutoCutsBase):
         loval = np.min(data)
         hival = np.max(data)
 
-        return (float(loval), float(hival))
+        return float(loval), float(hival)
 
 
 class Histogram(AutoCutsBase):
@@ -436,7 +439,7 @@ class Histogram(AutoCutsBase):
     def __init__(self, logger, usecrop=None, sample='crop',
                  full_px_limit=None, num_points=None,
                  pct=0.999, numbins=2048):
-        super(Histogram, self).__init__(logger)
+        super().__init__(logger)
 
         self.kind = 'histogram'
         self.sample = sample
@@ -460,7 +463,7 @@ class Histogram(AutoCutsBase):
 
         bnch = self.calc_histogram(data, pct=self.pct, numbins=self.numbins)
         loval, hival = bnch.loval, bnch.hival
-        return loval, hival
+        return float(loval), float(hival)
 
     def calc_cut_levels_data(self, data_np):
         """See subclass documentation."""
@@ -477,7 +480,7 @@ class Histogram(AutoCutsBase):
 
         bnch = self.calc_histogram(data, pct=self.pct, numbins=self.numbins)
         loval, hival = bnch.loval, bnch.hival
-        return loval, hival
+        return float(loval), float(hival)
 
     def calc_histogram(self, data, pct=1.0, numbins=2048):
         """Internal function used by this class."""
@@ -619,7 +622,7 @@ class StdDev(AutoCutsBase):
     def __init__(self, logger, usecrop=None, sample='grid',
                  full_px_limit=None, num_points=None,
                  hensa_lo=-1.5, hensa_hi=4.0):
-        super(StdDev, self).__init__(logger)
+        super().__init__(logger)
 
         self.kind = 'stddev'
         self.sample = sample
@@ -645,7 +648,7 @@ class StdDev(AutoCutsBase):
 
         loval, hival = self.calc_stddev(data, hensa_lo=self.hensa_lo,
                                         hensa_hi=self.hensa_hi)
-        return loval, hival
+        return float(loval), float(hival)
 
     def calc_cut_levels_data(self, data_np):
         """See subclass documentation."""
@@ -664,7 +667,7 @@ class StdDev(AutoCutsBase):
             return (0, 0)
         loval, hival = self.calc_stddev(data, hensa_lo=self.hensa_lo,
                                         hensa_hi=self.hensa_hi)
-        return loval, hival
+        return float(loval), float(hival)
 
     def calc_stddev(self, data, hensa_lo=-1.5, hensa_hi=4.0):
         """Internal function used by this class."""
@@ -714,7 +717,7 @@ class MedianFilter(AutoCutsBase):
         ]
 
     def __init__(self, logger, num_points=2000, length=5):
-        super(MedianFilter, self).__init__(logger)
+        super().__init__(logger)
 
         self.kind = 'median'
         self.num_points = num_points
@@ -725,14 +728,14 @@ class MedianFilter(AutoCutsBase):
         data = self.get_sample(image, num_points=self.num_points)
 
         loval, hival = self.calc_medianfilter(data, length=self.length)
-        return loval, hival
+        return float(loval), float(hival)
 
     def calc_cut_levels_data(self, data_np):
         """See subclass documentation."""
         data = self.get_sample_data(data_np, num_points=self.num_points)
 
         loval, hival = self.calc_medianfilter(data, length=self.length)
-        return loval, hival
+        return float(loval), float(hival)
 
     def calc_medianfilter(self, data, length=5):
         """Internal function used by this class."""
@@ -784,7 +787,7 @@ class ZScale(AutoCutsBase):
         ]
 
     def __init__(self, logger, contrast=0.25, num_points=None):
-        super(ZScale, self).__init__(logger)
+        super().__init__(logger)
 
         self.kind = 'zscale'
         self.contrast = contrast
@@ -796,15 +799,15 @@ class ZScale(AutoCutsBase):
 
         loval, hival = self.calc_zscale(data, contrast=self.contrast,
                                         num_points=self.num_points)
-        return loval, hival
+        return float(loval), float(hival)
 
     def calc_cut_levels_data(self, data_np):
         """See subclass documentation."""
-        cutout = self.get_sample_data(data_np)
+        cutout = self.get_sample_data(data_np, num_points=self.num_points)
 
         loval, hival = self.calc_zscale(cutout, contrast=self.contrast,
                                         num_points=self.num_points)
-        return loval, hival
+        return float(loval), float(hival)
 
     def calc_zscale(self, data, contrast=0.25, num_points=1000):
         """Internal function used by this class."""
@@ -827,7 +830,9 @@ class ZScale(AutoCutsBase):
 
         if samples.size == 0:
             return (0, 0)
-        loval, hival = zscale.zscale_samples(samples, contrast=contrast)
+
+        _interval = ZScaleInterval(samples.size, contrast=contrast)
+        loval, hival = _interval.get_limits(samples)
         return loval, hival
 
 
