@@ -20,11 +20,11 @@ class ImageViewAgg(ImageView.ImageViewBase):
 
         self.renderer = CanvasRenderer(self)
 
-    def update_image(self):
+    def update_widget(self):
         pass
 
     def configure_window(self, width, height):
-        self.configure_surface(width, height)
+        self.configure(width, height)
 
 
 class CanvasView(ImageViewAgg):
