@@ -151,7 +151,8 @@ class FitsViewer(object):
                   #'cap': 'ball',
                   }
         if kind in ('circle', 'rectangle', 'polygon', 'triangle',
-                    'righttriangle', 'ellipse', 'square', 'box'):
+                    'righttriangle', 'ellipse', 'square', 'box',
+                    'squarebox'):
             params['fill'] = fill
             params['fillalpha'] = alpha
 
