@@ -173,7 +173,8 @@ class FitsViewer(QtGui.QMainWindow):
                   'alpha': alpha,
                   }
         if kind in ('circle', 'rectangle', 'polygon', 'triangle',
-                    'righttriangle', 'ellipse', 'square', 'box'):
+                    'righttriangle', 'ellipse', 'square', 'box',
+                    'squarebox'):
             params['fill'] = fill
             params['fillalpha'] = alpha
 
