@@ -5,13 +5,9 @@
 # Please see the file LICENSE.txt for details.
 #
 import matplotlib
-# GTK3Cairo backend is slow to redraw compared to GTK3Agg!
-# matplotlib.use('GTK3Cairo')
-# from matplotlib.backends.backend_gtk3cairo import (FigureCanvasGTK3Cairo
-#                                                    as FigureCanvas)  # noqa
-matplotlib.use('GTK3Agg')
-from matplotlib.backends.backend_gtk3agg import (FigureCanvasGTK3Agg
-                                                 as FigureCanvas)  # noqa
+matplotlib.use('GTK3Cairo')
+from matplotlib.backends.backend_gtk3cairo import (FigureCanvasGTK3Cairo
+                                                   as FigureCanvas)  # noqa
 from ginga.gtk3w import Widgets  # noqa
 
 
@@ -36,5 +32,3 @@ class PlotWidget(Widgets.WidgetBase):
         self.logger.debug("canvas resized to %dx%d" % (wd, ht))
         fig = self.plot.get_figure()
         fig.set_size_inches(float(wd) / fig.dpi, float(ht) / fig.dpi)
-
-# END
