@@ -9,8 +9,6 @@ import math
 import os.path
 import uuid
 
-import yaml
-
 from ginga.misc import Bunch, Callback
 from ginga.gw import Widgets, Viewers
 from ginga.util import json
@@ -730,22 +728,10 @@ class Desktop(Callback.Callbacks):
 
             if rec.kind in ('ws',):
                 # record ws type
-                # TODO: record positions of subwindows
                 wstype = rec.ws.wstype
                 d = dict(wstype=wstype)
-                cd = dict()
-                for child in rec.ws.nb.get_children():
-                    if child.get_widget() is None:
-                        continue
-                    title = child.extdata.get('tab_title', None)
-                    mdi_pos = child.extdata.get('mdi_pos', (0, 0))
-                    size = child.extdata.get('mdi_size', None)
-                    if size is None:
-                        size = child.get_size()
-                    if title is not None:
-                        cd[title] = dict(title=title, mdi_pos=mdi_pos,
-                                         size=size)
-                d['child_attribs'] = cd
+                cfg_d = rec.ws.get_configuration()
+                d['child_attribs'] = cfg_d['tabs']
                 rec.params.update(d)
 
             if rec.kind == 'top':
@@ -764,7 +750,11 @@ class Desktop(Callback.Callbacks):
         # write layout
         _n, ext = os.path.splitext(lo_file)
         with open(lo_file, 'w') as out_f:
-            if ext.lower() in ['.yml', '.yaml']:
+            if len(ext) == 0:
+                import pprint
+                pprint.pprint(layout, out_f)
+            elif ext.lower() in ['.yml', '.yaml']:
+                import yaml
                 out_f.write(yaml.dump(layout, indent=2))
             else:
                 out_f.write(json.dumps(layout, indent=2))
@@ -775,7 +765,12 @@ class Desktop(Callback.Callbacks):
             buf = in_f.read()
 
         _n, ext = os.path.splitext(lo_file)
-        if ext.lower() in ['.yml', '.yaml']:
+        if len(ext) == 0:
+            # older, python format
+            import ast
+            layout = ast.literal_eval(buf)
+        elif ext.lower() in ['.yml', '.yaml']:
+            import yaml
             layout = yaml.safe_load(buf)
         else:
             layout = json.loads(buf)
@@ -1041,6 +1036,29 @@ class Workspace(Widgets.WidgetBase):
     def remove_tab(self, child):
         self.nb.remove(child)
 
+    def get_configuration(self):
+        """Return the configuration of this workspace as a dictionary."""
+        cd = dict()
+        for child in self.nb.get_children():
+            if child.get_widget() is None:
+                continue
+            title = child.extdata.get('tab_title', None)
+            mdi_pos = child.extdata.get('mdi_pos', (0, 0))
+            size = child.extdata.get('mdi_size', None)
+            if size is None:
+                size = child.get_size()
+            if title is not None:
+                cd[title] = dict(title=title, mdi_pos=mdi_pos,
+                                 size=size)
+        res = dict(wsname=self.name, wstype=self.wstype, tabs=cd)
+        return res
+
+    def record_positions(self):
+        """Record our placement and size of child windows in the workspace."""
+        res = self.get_configuration()
+        self.child_catalog.update(res['tabs'])
+        return res
+
 
 class SymmetricGridWidget(Widgets.GridBox):
     """Custom widget for grid-type workspace that has the API of a
