@@ -7,6 +7,9 @@ if tkname == 'qt':
 elif tkname == 'gtk3':
     from ginga.gtk3w.Viewers import *  # noqa
 
+elif tkname == 'gtk4':
+    from ginga.gtk4w.Viewers import *  # noqa
+
 elif tkname == 'pg':
     from ginga.web.pgw.Viewers import *  # noqa
 
