@@ -22,7 +22,7 @@ from ginga.misc.Bunch import Bunch
 from ginga.misc import Task, ModuleManager, Settings, log
 import ginga.version as version
 import ginga.toolkit as ginga_toolkit
-from ginga.util import paths, rgb_cms, compat, loader
+from ginga.util import paths, rgb_cms, compat, loader, grc
 
 # Catch warnings
 logging.captureWarnings(True)
@@ -351,6 +351,9 @@ class ReferenceViewer(object):
                      help="Use OpenGL acceleration")
         add_argument("--plugins", dest="plugins", metavar="NAMES",
                      help="Specify additional plugins to load")
+        add_argument("--rcport", dest="rc_port", type=int,
+                     default=None, metavar="NUM",
+                     help="Use PORT for Ginga Remote Control plugin")
         add_argument("--sep", dest="separate_channels", default=False,
                      action="store_true",
                      help="Load files in separate channels")
@@ -402,6 +405,10 @@ class ReferenceViewer(object):
                         self.appname, self.basedir, str(e)))
                 logger.warning("Preferences will not be able to be saved")
 
+        if options.rc_port is not None:
+            # user specified a custom Remote Control port
+            grc.default_rc_port = options.rc_port
+
         # Set up preferences
         prefs = Settings.Preferences(basefolder=self.basedir, logger=logger)
         settings = prefs.create_category('general')
@@ -412,7 +419,7 @@ class ReferenceViewer(object):
                               WCSpkg='choose', FITSpkg='choose',
                               suppress_fits_warnings=False,
                               recursion_limit=2000,
-                              num_threads=30,
+                              num_threads=4,
                               icc_working_profile=None,
                               font_scaling_factor=None,
                               save_layout=True,
@@ -435,6 +442,7 @@ class ReferenceViewer(object):
 
         # Create the dynamic module manager
         mm = ModuleManager.ModuleManager(logger)
+        sys.meta_path.append(mm)
 
         # what is the dynamic config file to load
         app_config = "{}_config".format(self.appname)
@@ -529,11 +537,11 @@ class ReferenceViewer(object):
 
         ev_quit = threading.Event()
         # Create and start thread pool
-        num_threads = settings.get('num_threads', 30)
+        num_threads = settings.get('num_threads', 4)
         if options.numthreads is not None:
             num_threads = options.numthreads
-        thread_pool = Task.ThreadPool(num_threads, logger,
-                                      ev_quit=ev_quit)
+        thread_pool = Task.ThreadPool(numthreads=num_threads, logger=logger,
+                                      minthreads=1, ev_quit=ev_quit)
         thread_pool.startall()
 
         # Create the Ginga main object
