@@ -15,6 +15,8 @@ import xmlrpc.server as SimpleXMLRPCServer  # nosec
 # undefined passed value--for a data type that cannot be converted
 undefined = '#UNDEFINED'
 
+default_rc_port = 11771
+
 
 class _ginga_proxy(object):
 
@@ -194,15 +196,19 @@ class RemoteClient(object):
 
 class RemoteServer(object):
 
-    def __init__(self, obj, host='localhost', port=9000, ev_quit=None,
+    def __init__(self, obj, host='localhost', port=None, ev_quit=None,
                  logger=None):
         super(RemoteServer, self).__init__()
 
         self.robj = obj
         # What port to listen for requests
+        if port is None:
+            port = default_rc_port
         self.port = port
         # If blank, listens on all interfaces
         self.host = host
+        # this holds the RPC server
+        self.server = None
 
         if logger is None:
             logger = log.get_logger(null=True)
@@ -226,11 +232,12 @@ class RemoteServer(object):
             self.server.serve_forever(poll_interval=0.1)
 
     def stop(self):
-        self.server.shutdown()
-        # TODO: unfortunate that we have to reach inside the implementation
-        # of socketserver.  It appears that when you shut down the server it
-        # does *not* close the socket.
-        self.server.socket.close()
+        if self.server is not None:
+            self.server.shutdown()
+            # TODO: unfortunate that we have to reach inside the implementation
+            # of socketserver.  It appears that when you shut down the server it
+            # does *not* close the socket.
+            self.server.socket.close()
         self.server = None
 
     def restart(self):
